package es.um.atica.cursoj2ee.security.authentication;

import static org.jboss.seam.ScopeType.SESSION;
import static org.jboss.seam.annotations.Install.FRAMEWORK;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.model.SelectItem;

import org.jboss.seam.Component;
import org.jboss.seam.annotations.Install;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;
import org.jboss.seam.annotations.intercept.BypassInterceptors;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.SeamResourceBundle;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;
import org.jboss.seam.log.Logging;
import org.umu.atica.servicios.pci.sestertium.ServicioSestertiumUmu;

import buscador.servicios.BuscadorServiciosAtica;
import buscador.servicios.exceptions.ServiceNotFoundException;
import es.um.atica.seam.components.FundeWebManagerBean;
import es.um.atica.seam.security.CredentialsAdapter;
import es.um.atica.seam.security.UmuIdentity;
import es.um.atica.seam.security.authentication.credentials.CredentialsCertificateUmu;
import es.um.atica.seam.security.authentication.credentials.CredentialsUmu;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactory;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactoryCard;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactoryCertificate;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactoryNif;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactoryRadius;
import es.um.atica.seam.security.authentication.factories.AuthenticationFactorySSO;
import es.um.atica.seam.security.authentication.method.AuthenticationMethod;

@Name("authenticationManagerBean")
@Scope(SESSION)
@Install(precedence = FRAMEWORK)
@BypassInterceptors
@Startup
public class AuthenticationManagerBean extends FundeWebManagerBean implements Serializable {

	/**
	 * serialVersionUID generado automaticamente
	 */
	private static final long serialVersionUID = -6064182119922723132L;

	/** Logger de la clase */
	private static final Log LOG = Logging.getLog(AuthenticationManagerBean.class);

	protected SelectItem[] selectItemsAutentication;

	/** Credencial actual */
	protected CredentialsAdapter credentialsAdapter;

	protected AuthenticationType authenticationType;

	public enum AuthenticationType {
		RADIUS, CARD, CERTIFICATE, NIF, SSO
	}

	public AuthenticationManagerBean() { // Por defecto RADIUS
		this.credentialsAdapter = (CredentialsAdapter) this.getCredentials();
		this.authenticationType = AuthenticationType.RADIUS;
		this.activateCredentialsUmu();

		int idx = 0;
		selectItemsAutentication = new SelectItem[AuthenticationType.values().length];
		for (AuthenticationType type : AuthenticationType.values()) {
			selectItemsAutentication[idx++] = new SelectItem(type.name(), getAuthenticationTypeLabel(type));
		}
	}

	public void activateCredentialsUmu() {
		LOG.info("Entrar en activateCredentialsUmu: #0", this.authenticationType.name());
		this.credentialsAdapter.setCredentialsUmu(getFactoria(this.authenticationType).createCredentials());
	}

	/**
	 * Mtodo para activar una credencial.<br />
	 * Si la que se desea activar, es la que est actualmente, no se hace nada y se devuelve false. En otro caso se
	 * devolver true.
	 * 
	 * @param credencial
	 *            Clase de Credencial a activar.
	 * @return Si => se creo una nueva credencial. No => ya estaba esa misma credencial activa.
	 */
	public boolean activateCredentialsUmu(AuthenticationType authenticationType) {
		LOG.info("Entrar en activateCredentialsUmu: #0", (authenticationType != null ? authenticationType.name() : ""));
		if ((this.getCredentialsUmu() != null) && (this.authenticationType == authenticationType)) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("La credencial actual y la pedida son iguales, luego no se crear una nueva: #0.",
						this.authenticationType);
			}
			return false;
		}
		if (authenticationType != null) {
			this.setAuthenticationType(authenticationType);
		}
		else { // Por defecto Radius
			this.setAuthenticationType(AuthenticationType.RADIUS);
		}
		this.credentialsAdapter.setCredentialsUmu(getFactoria(this.authenticationType).createCredentials());
		return true;
	}

	public AuthenticationMethod getAuthenticationMethod() {
		return this.getFactoria(this.authenticationType).createAuthenticationMethod();
	}

	protected AuthenticationFactory getFactoria(AuthenticationType authenticationType) {
		if (this.authenticationType == null) {
			activateCredentialsUmu(AuthenticationType.RADIUS);
		}
		switch (this.authenticationType) {
			case CARD: {
				return new AuthenticationFactoryCard();
			}
			case CERTIFICATE: {
				// Limpiamos el error en la firma para preparar la autenticacion
				// this.errorFirma = 0;
				return new AuthenticationFactoryCertificate();
			}
			case NIF: {
				return new AuthenticationFactoryNif();
			}
			// case RADIUS: {
			// return new AuthenticationFactoryRadius();
			// }
			case SSO: { // case RADIUS
				return new AuthenticationFactorySSO();
			}
			default: {
				return new AuthenticationFactoryRadius();
			}
		}
	}

	protected String getAuthenticationTypeLabel(AuthenticationType authenticationType) {
		ResourceBundle srb = SeamResourceBundle.getBundle();

		try {
			switch (authenticationType) {
				case CARD: {
					return srb.getString("label.tipo_acceso_tarjeta");
				}
				case CERTIFICATE: {
					return srb.getString("label.tipo_acceso_certificado");
				}
				case NIF: {
					return srb.getString("label.tipo_acceso_nif");
				}
				case RADIUS: {
					return srb.getString("label.tipo_acceso_correo");
				}
				case SSO: {
					return srb.getString("label.tipo_acceso_sso");
				}
				default:
					return srb.getString("tipo no identificado");
			}
		} catch (MissingResourceException mre) {
			LOG.error("Error al obtener las etiquetas para los tipos de autenticacion.", mre);
		}
		return "";
	}

	/**
	 * Obtiene la credencial actual.
	 */
	public CredentialsUmu getCredentialsUmu() {
		return this.credentialsAdapter.getCredentialsUmu();
	}

	public AuthenticationType getAuthenticationType() {
		return authenticationType;
	}

	public void setAuthenticationType(AuthenticationType authenticationType) {
		LOG.debug("Entra en setAuthenticationType: #0 - #1", authenticationType.hashCode(), authenticationType.name());
		this.authenticationType = authenticationType;
	}

	public boolean isRadiusAuthentication() {
		return this.authenticationType == AuthenticationType.RADIUS;
	}

	public boolean isCardAuthentication() {
		return this.authenticationType == AuthenticationType.CARD;
	}

	public boolean isCertificateAuthentication() {
		return this.authenticationType == AuthenticationType.CERTIFICATE;
	}

	public boolean isNifAuthentication() {
		return this.authenticationType == AuthenticationType.NIF;
	}

	public boolean isSsoAuthentication() {
		return this.authenticationType == AuthenticationType.SSO;
	}

	public SelectItem[] getSelectItemsAutentication() {
		return selectItemsAutentication;
	}

	/* ----------------- AUTENTICACION CON TARJETA --------------------- */
	public String getSestertiumApplet() {
		LOG.debug("Obteniendo el codigo HTML del APPLET Sestertium.");
		try {
			ServicioSestertiumUmu servicioSestertiumUmu = (ServicioSestertiumUmu) BuscadorServiciosAtica
					.obtenerServicio(ServicioSestertiumUmu.ID);
			return servicioSestertiumUmu.getSestertiumApplet();
		} catch (ServiceNotFoundException snfe) {
			LOG.error("Error al buscar el servicio de Radius", snfe);
			this.addFacesMessageFromResourceBundle(Severity.ERROR, "authentication.card.applet.error.message");
			return "";
		} catch (SQLException se) {
			LOG.error("Error al obtener el HTML de Appelt Sestertium:", se);
			this.addFacesMessageFromResourceBundle(Severity.ERROR, "authentication.card.applet.error.message");
			return "";
		} catch (Throwable t) {
			LOG.error("Error al obtener el HTML de Appelt Sestertium:", t);
			this.addFacesMessageFromResourceBundle(Severity.ERROR, "authentication.card.applet.error.message");
			return "";
		}
	}

	/* ----------------- AUTENTICACION CON CERTIFICADO --------------------- */
	/**
	 * Mtodo para obtener el XML con el Reto
	 * 
	 * @return XML con el Reto
	 */
	public String getRetoXmlFuente() {
		LOG.debug("Entra en getRetoXmlFuente");
		return ((CredentialsCertificateUmu) this.getCredentialsUmu()).getRetoXml();
	}

	/**
	 * Mtodo para actualizar el XML con el Reto
	 * 
	 * @param retoXml
	 *            XML con el Reto a cargar
	 */
	public void setRetoXmlFuente(String retoXmlFuente) {
		LOG.debug("Entra en setRetoXmlFuente");
		((CredentialsCertificateUmu) this.getCredentialsUmu()).setRetoXml(retoXmlFuente);
	}

	public String getRetoXmlFirmado() {
		return ((CredentialsCertificateUmu) this.getCredentialsUmu()).getRetoXmlFirmado();
	}

	public void setRetoXmlFirmado(String retoXmlFirmado) {
		((CredentialsCertificateUmu) this.getCredentialsUmu()).setRetoXmlFirmado(retoXmlFirmado);
	}

	@Observer(UmuIdentity.EVENT_AUTHENTICATING_BY_CAS)
	public void activarAuthenticacionSSO() {
		LOG.debug("Entra en activarAuthenticacionSSO");
		this.authenticationType = AuthenticationType.SSO;
		this.activateCredentialsUmu();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.um.atica.util.FundeWebManager#getLog()
	 */
	@Override
	protected Log getLog() {
		return LOG;
	}

	public static AuthenticationManagerBean instance() {
		if (!Contexts.isSessionContextActive()) {
			throw new IllegalStateException("no session context active");
		}
		return (AuthenticationManagerBean) Component.getInstance(AuthenticationManagerBean.class);
	}

	public String getErrorFirma() {
		return "0";
	}
	
	public void setErrorFirma() {
		
	}
}
